package tsyliveaudiorecord.live_audio_record;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.graphics.Color;
import android.media.browse.MediaBrowser;
import android.os.Binder;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.service.media.MediaBrowserService;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.core.app.NotificationCompat;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Queue;

import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Consumer;

@RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
public class AudioRecordService extends Service {

    private static final String LOG_TAG = "RecordingService";



    public AudioRecorder audioRecorder;
    private NotificationManager mNotificationManager;
    private static final int NOTIFY_ID = 100;

    public  AudioSaveHelper audioSaveHelper;
    public AudioRecordingDbmHandler handler;
    private ServiceBinder mIBinder;
    private AudioRecorder.RecordTime lastUpdated;
    private boolean mIsClientBound = false;
    int sampleRate = 16000;
    String filePath = "2222";
    Boolean startForeground = false;
    Boolean isWriteFile = false;
    String mExtension ="" ;
    int bufferSize = 3000 ;


    @Override public IBinder onBind(Intent intent) {
        mIsClientBound = true;
        return mIBinder;
    }

//    @Nullable
//    @Override
//    public BrowserRoot onGetRoot(@NonNull String clientPackageName, int clientUid, @Nullable Bundle rootHints) {
//        return null;
//    }
//
//    @Override
//    public void onLoadChildren(@NonNull String parentId, @NonNull Result<List<MediaBrowser.MediaItem>> result) {
//
//    }

    public boolean isRecording() {
        return audioRecorder.isRecording();
    }

    @Override
    public void onCreate() {
        super.onCreate();
        Log.d("kiemtra","hello onCreate 4");

        handler = new AudioRecordingDbmHandler();
        audioSaveHelper = new AudioSaveHelper();

        audioRecorder = new AudioRecorder(audioSaveHelper);
        handler.addRecorder(audioRecorder);

        mIBinder = new ServiceBinder();
        mNotificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);

    }



    public AudioRecordingDbmHandler getHandler() {
        return handler;
    }
    Handler timerHandler;
    Runnable timerRunnable;
    @Override public int onStartCommand(Intent intent, int flags, int startId) {

        if (intent.getAction() != null) {
            switch (intent.getAction()) {
                case AppConstants.ACTION_PAUSE:
                    pauseRecord();
                    break;
                case AppConstants.ACTION_RESUME:
                    resumeRecord();
                    break;
                case AppConstants.ACTION_STOP:
                    if (!mIsClientBound) {
                        stopSelf();
                    }
            }
            if (mIsClientBound) {
                intent.putExtra(AppConstants.ACTION_IN_SERVICE, intent.getAction());
                intent.setAction(AppConstants.ACTION_IN_SERVICE);
                LocalBroadcastManager.getInstance(this).sendBroadcast(intent);
            }
        }
        else {
            filePath = intent.getStringExtra("mFilePath");
            sampleRate = intent.getIntExtra("mSampleRate",16000);
            startForeground = intent.getBooleanExtra("startForeground",false);
            isWriteFile = intent.getBooleanExtra("isWriteFile",false);
            timerHandler = new Handler();

            Log.d("kiemtra","truoc khi start");
            startRecording();
           timerRunnable  = new Runnable() {

               @Override
               public void run() {
                   if(audioRecorder.qe !=null && audioRecorder.qe.size() > 0){
                       HashMap<String, Object> x  = audioRecorder.qe.poll();
                       if(x != null) {
                           Intent intent = new Intent("GPSLocationUpdates");
                           // You can also include some extra data.
                           intent.putExtra("current", x);
                           LocalBroadcastManager.getInstance(getApplicationContext()).sendBroadcast(intent);
//                       audioSaveHelper.onDataReady(x.get("buffer"));
                       }
                   }

                   timerHandler.postDelayed(this, 32);
               }
           };

            timerRunnable.run();

            if(startForeground){
                startForeground(101, createNotification(new AudioRecorder.RecordTime()));
            }
           // mNotificationManager.cancel(0);


        }
        return START_NOT_STICKY;
    }
    @RequiresApi(Build.VERSION_CODES.O)
    private String createNotificationChannel(String channelId, String channelName){
        NotificationChannel chan = new NotificationChannel(channelId,
                channelName, NotificationManager.IMPORTANCE_NONE);
        chan.setLightColor(Color.BLUE);
        chan.setLockscreenVisibility(Notification.VISIBILITY_PRIVATE);
        mNotificationManager.createNotificationChannel(chan);
        return channelId;
    }





    @Override public void onDestroy() {
        super.onDestroy();
        Log.d("kiemtra","onDestroy");
        if (isRecording()) {
            stopRecodingAndRelease();
        }
    }

    private void stopRecodingAndRelease() {
        audioRecorder.finishRecord();
        timerHandler.removeCallbacks(timerRunnable);
        timerHandler = null;
        timerRunnable = null;
        handler.stop();
        handler = null;
        audioRecorder = null;
        audioSaveHelper = null;
        mIBinder = null;
    }

    private void startRecording() {
        audioRecorder.startRecord(
                sampleRate ,filePath,isWriteFile);
        Log.d("kiemtra","" + sampleRate);
        handler.startDbmThread();
//       audioRecorder.subscribeTimer(this::updateNotification);
    }

    private void updateNotification(AudioRecorder.RecordTime recordTime) {
        //mNotificationManager.notify(NOTIFY_ID, createNotification(recordTime));
    }

    private Notification createNotification(AudioRecorder.RecordTime recordTime) {
        Log.d("kiemtra","createNotification");
        lastUpdated = recordTime;
        String channeID = "" ;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O){
            channeID = createNotificationChannel("tsyliveaudiorecord.live_audio_record","My Background Service");
        }
        NotificationCompat.Builder notificationBuilder = new NotificationCompat.Builder(this ,channeID);
        Notification notification = notificationBuilder.setOngoing(true)
                .setSmallIcon(R.drawable.ic_media_record)
                .setContentTitle(getString(R.string.notification_recording))
                .setContentText("Recording...........")
                .setPriority(NotificationManager.IMPORTANCE_MIN)
                .setCategory(Notification.CATEGORY_SERVICE)
                .build();
//        if (audioRecorder.isPaused()) {
//            notificationBuilder.addAction(R.drawable.ic_media_record, getString(R.string.resume_recording_button),
//                    getActionIntent(AppConstants.ACTION_RESUME));
//        } else {
//            notificationBuilder.addAction(R.drawable.ic_media_pause, getString(R.string.pause_recording_button),
//                    getActionIntent(AppConstants.ACTION_PAUSE));
//        }


        return notification;
    }


    public void pauseRecord() {
        audioRecorder.pauseRecord();

    }
    public boolean isPaused() {
        return audioRecorder.isPaused();
    }

    public void resumeRecord() {
        audioRecorder.resumeRecord();
    }

    public class ServiceBinder extends Binder {
        public AudioRecordService getService() {
            return AudioRecordService.this;
        }
    }

    private PendingIntent getActionIntent(String action) {
        Intent pauseIntent = new Intent(this, AudioRecordService.class);
        pauseIntent.setAction(action);
        return PendingIntent.getService(this, 100, pauseIntent, 0);
    }

    @Override public boolean onUnbind(Intent intent) {
        mIsClientBound = false;
        return true;
    }

    @Override public void onRebind(Intent intent) {
        mIsClientBound = true;
    }
}
