import 'dart:async';
import 'dart:io';
import 'package:grpc/grpc.dart';
import '../proto/generated/cloud_speech.pbgrpc.dart' as cloudSpeech;
import '../vais_speech_audio.dart';

class VaisStreamingController {
  cloudSpeech.StreamingRecognizeRequest? gRpcRequestStream;
  cloudSpeech.StreamingRecognitionConfig? gRpcConfigStream;
  cloudSpeech.RecognitionConfig? gRpcConfig;
  late cloudSpeech.SpeechClient gRpcStub;
  String? _token;
  String grpcAddress;
  int port;
  static int frame_size = Platform.isAndroid ? 1280 : 3200;
  static double frame_duration = (( frame_size / 640) * 0.02);
  StreamController<cloudSpeech.StreamingRecognizeRequest?> gRpcRequestStreamController = StreamController();

  String? get token => _token;

  VaisStreamingController(this._token,this.grpcAddress,this.port);

  void initialize() {
    final channel = ClientChannel(grpcAddress,
        port: port,
        options: const ChannelOptions(credentials: const ChannelCredentials.insecure()));
    gRpcStub = cloudSpeech.SpeechClient(channel, options: CallOptions(metadata: {"api-key" : _token!}));
    gRpcConfig = cloudSpeech.RecognitionConfig();
    gRpcConfigStream = cloudSpeech.StreamingRecognitionConfig.create();
    gRpcRequestStream = cloudSpeech.StreamingRecognizeRequest.create();
  }

  void dispose() {
    gRpcRequestStreamController.close();
  }
  String totalOutput ="";
  void requestBeforeStartAudioRecord(bool singleUtterance, VaisSpeechCallback? listener,String modelParamText) {
    gRpcConfig!.clear();
    gRpcConfig!.encoding = cloudSpeech.RecognitionConfig_AudioEncoding.LINEAR16;
    gRpcConfig!.sampleRateHertz =  16000;

    var modelParam = new cloudSpeech.ModelParam();
    modelParam.graph = modelParamText;
    gRpcConfig!.modelParam = modelParam;

    gRpcConfigStream!.clear();
    gRpcConfigStream!.config = gRpcConfig!;
    gRpcConfigStream!.interimResults = true;
    gRpcConfigStream!.singleUtterance = singleUtterance;

    gRpcRequestStream!.clear();
    gRpcRequestStream!.streamingConfig = gRpcConfigStream!;

    if (gRpcRequestStreamController.isClosed) {
      gRpcRequestStreamController = StreamController();
    }
    gRpcRequestStreamController.sink.add(gRpcRequestStream);
    listener?.onConnectionChanged!(true);

    gRpcStub.streamingRecognize(gRpcRequestStreamController.stream).listen((data) {
      var result = data.results[0];
      final String output = result.alternatives[0].transcript;
      print("Success >>>>>>>>>>>>>> $output");

      listener!.onConnectionChanged!(true);
      int stime = 0;
      int etime = 0;
      String audioPath = "";
      String audioName = "";
      try{
        if(result.isFinal){
          print('audio file name nay');
          print(result.audioFilename);
          totalOutput = totalOutput + ' '+output;

          if(result.audioFilename != null && result.audioFilename.isNotEmpty){
            audioPath = result.audioFilename.substring(result.audioFilename.lastIndexOf('|')+1);
            String endTime= result.audioFilename.substring(result.audioFilename.lastIndexOf('__')+2).replaceAll('|'+audioPath, '');
            String startTime = result.audioFilename.substring(result.audioFilename.indexOf('__')+2).replaceAll('__'+endTime+'|'+audioPath, '');
            print(startTime);
            stime  = ((int.parse(startTime) / frame_size ).toDouble() * frame_duration * 1000).toInt();
            etime  = ((int.parse(endTime) / frame_size ).toDouble() * frame_duration * 1000).toInt();
            audioName = result.audioFilename.replaceAll('__'+startTime+'__'+endTime+'|'+audioPath, '');
            print(audioName);
            print(stime);
            print(etime);
            print(audioPath);
          }

          listener.onGeneratedSpeechText!(totalOutput,output,result.isFinal,audioName,stime,etime,audioPath);
        }
        else{
          listener.onGeneratedSpeechText!(totalOutput + ' '+output,output,result.isFinal,audioName,stime,etime,audioPath);
        }
      }catch(e){
        print(e);
        print('error');
      }

    }, onError: (error) {
      print("Error $error");
      listener!.onConnectionChanged!(false);
    }, onDone: () {
      print("Done");
      listener!.onConnectionChanged!(false);
    });
  }

  void streamingAudioRecord(List<int> buffers) {
    print('streamingAudioRecord');
    print(buffers.length);
    gRpcRequestStream!.clear();
    gRpcRequestStream!.audioContent = buffers;
    gRpcRequestStreamController.add(gRpcRequestStream);
  }

  bool isStreamingClosed() {
    return gRpcRequestStreamController.isClosed;
  }
}